// at_material.cc: ACIS material attrib implementation.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: at_material.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (at_material,
       "$Id: at_material.cc 1 2005-08-10 23:35:07Z frank $");

#include "acis/acis.hxx"

#include "acis/at_material.hxx"

// Define macros for this attribute and its parent, to simplify 
// later stuff and make it suitable for making into macros. 
#define THIS() ATTRIB_MATERIAL
#define THIS_LIB NONE
#define PARENT() ATTRIB_SIL
#define PARENT_LIB KERN

// Identifier used externally to identify a particular entity 
// type. This is used in the save/restore system for translating 
// to/from external file format. 
#define ATTRIB_MATERIAL_NAME "material" 
// Implement the standard attribute functions. 

ATTRIB_DEF("material_attribute") 

  char *str = new char[255];
  const GLfloat *c = get_ambient ();
  snprintf (str, 255, "%f %f %f %f", *c, c[1], c[2], c[3]);
  debug_string ("ambient", str, fp); 
  c = get_diffuse ();
  snprintf (str, 255, "%f %f %f %f", *c, c[1], c[2], c[3]);
  debug_string ("diffuse", str, fp); 
  c = get_specular ();
  snprintf (str, 255, "%f %f %f %f", *c, c[1], c[2], c[3]);
  debug_string ("specular", str, fp); 
  c = get_emission ();
  snprintf (str, 255, "%f %f %f %f", *c, c[1], c[2], c[3]);
  debug_string ("emission", str, fp); 
  delete str;

  debug_real("shininess", (double)*get_shininess (), fp); 

SAVE_DEF 

  const GLfloat *c = get_ambient ();
  double dc = (double)*c;
  write_real (dc);
  dc = (double)c[1];
  write_real (dc);
  dc = (double)c[2];
  write_real (dc);
  dc = (double)c[3];
  write_real (dc);

  c = get_diffuse ();
  dc = (double)*c;
  write_real (dc);
  dc = (double)c[1];
  write_real (dc);
  dc = (double)c[2];
  write_real (dc);
  dc = (double)c[3];
  write_real (dc);

  c = get_specular ();
  dc = (double)*c;
  write_real (dc);
  dc = (double)c[1];
  write_real (dc);
  dc = (double)c[2];
  write_real (dc);
  dc = (double)c[3];
  write_real (dc);

  c = get_emission ();
  dc = (double)*c;
  write_real (dc);
  dc = (double)c[1];
  write_real (dc);
  dc = (double)c[2];
  write_real (dc);
  dc = (double)c[3];
  write_real (dc);

  c = get_shininess ();
  dc = (double)*c;
  write_real (dc);

RESTORE_DEF 

  set ((GLfloat)read_real (), (GLfloat)read_real (), 
       (GLfloat)read_real (), (GLfloat)read_real (),
       (GLfloat)read_real (), (GLfloat)read_real (), 
       (GLfloat)read_real (), (GLfloat)read_real (),
       (GLfloat)read_real (), (GLfloat)read_real (), 
       (GLfloat)read_real (), (GLfloat)read_real (),
       (GLfloat)read_real (), (GLfloat)read_real (), 
       (GLfloat)read_real (), (GLfloat)read_real (),
       (GLfloat)read_real ());

COPY_DEF 

  const GLfloat *a = from->get_ambient ();
  const GLfloat *d = from->get_diffuse ();
  const GLfloat *s = from->get_specular ();
  const GLfloat *e = from->get_emission ();
  const GLfloat *ss = from->get_shininess ();
  set (a[0], a[1], a[2], a[3],
       d[0], d[1], d[2], d[3],
       s[0], s[1], s[2], s[3],
       e[0], e[1], e[2], e[3],
       *ss);

SCAN_DEF 
  // (no specific pointer data) 

FIX_POINTER_DEF 
  // (no specific pointer data) 

TERMINATE_DEF 

// make a material attribute 
ATTRIB_MATERIAL::ATTRIB_MATERIAL (ENTITY *owner,
				  double ar, double ag, double ab, double aa,
				  double dr, double dg, double db, double da,
				  double sr, double sg, double sb, double sa,
				  double er, double eg, double eb, double ea,
				  double ss) 
  : ATTRIB_SIL (owner),
    Material ((GLfloat)ar, (GLfloat)ag, (GLfloat)ab, (GLfloat)aa,
	      (GLfloat)dr, (GLfloat)dg, (GLfloat)db, (GLfloat)da,
	      (GLfloat)sr, (GLfloat)sg, (GLfloat)sb, (GLfloat)sa,
	      (GLfloat)er, (GLfloat)eg, (GLfloat)eb, (GLfloat)ea,
	      (GLfloat)ss)
{
  ;
}

// Set the member data. 
void ATTRIB_MATERIAL::set_material (double ar, double ag, double ab, double aa,
				    double dr, double dg, double db, double da,
				    double sr, double sg, double sb, double sa,
				    double er, double eg, double eb, double ea,
				    double ss)
{
  backup ();
  set ((GLfloat)ar, (GLfloat)ag, (GLfloat)ab, (GLfloat)aa,
       (GLfloat)dr, (GLfloat)dg, (GLfloat)db, (GLfloat)da,
       (GLfloat)sr, (GLfloat)sg, (GLfloat)sb, (GLfloat)sa,
       (GLfloat)er, (GLfloat)eg, (GLfloat)eb, (GLfloat)ea,
       (GLfloat)ss);
  return;
}

// Virtual function called when an owner entity is being split. 
void ATTRIB_MATERIAL::split_owner (ENTITY* new_ent) 
{ 
  const GLfloat *a = get_ambient ();
  const GLfloat *d = get_diffuse ();
  const GLfloat *s = get_specular ();
  const GLfloat *e = get_emission ();
  const GLfloat *ss = get_shininess ();
  new ATTRIB_MATERIAL (new_ent,
		       (double)a[0], (double)a[1], (double)a[2], (double)a[3],
		       (double)d[0], (double)d[1], (double)d[2], (double)d[3],
		       (double)s[0], (double)s[1], (double)s[2], (double)s[3],
		       (double)e[0], (double)e[1], (double)e[2], (double)e[3],
		       (double)*ss);
  return;
}

// Virtual function called when two entities are to be merged. 
void ATTRIB_MATERIAL::merge_owner (ENTITY* other_ent, logical delete_owner)
{
  // If the owner of this attribute is going to be deleted, and
  // there is no color attached to the other entity, then we
  // transfer ourself to that other entity.
  if (delete_owner) {
    ATTRIB* other_att = find_attrib(other_ent,
				    ATTRIB_SIL_TYPE,
				    ATTRIB_MATERIAL_TYPE);
    if(other_att == NULL) {
      // No color on other entity, so transfer ourselves
      move(other_ent);
    }
  }
  return;
}

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
