// light.cc: LIGHT class implementation.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: light.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (light, "$Id: light.cc 1 2005-08-10 23:35:07Z frank $");

#include "acis/acis.hxx"

#include <kernel/kernel_thread_ctx.hxx>

#include "acis/light.hxx"

// Include the standard member functions via the usual macros. 
// First declare our class name and that of our parent. 
#define THIS() LIGHT
#define THIS_LIB NONE 
#define PARENT() ENTITY_SIL
#define PARENT_LIB NONE

// External name for this entity type, for save/restore and 
// general communications with the user. 
#define LIGHT_NAME "light" 

ENTITY_DEF (LIGHT_NAME) 

  char *str = new char[255];
  const GLfloat *c = get_ambient ();
  snprintf (str, 255, "%g %g %g %g", c[0], c[1], c[2], c[3]);
  debug_string ("ambient", str, fp); 
  c = get_diffuse ();
  snprintf (str, 255, "%g %g %g %g", c[0], c[1], c[2], c[3]);
  debug_string ("diffuse", str, fp); 
  c = get_specular ();
  snprintf (str, 255, "%g %g %g %g", c[0], c[1], c[2], c[3]);
  debug_string ("specular", str, fp); 
  c = get_position ();
  snprintf (str, 255, "%g %g %g %g", c[0], c[1], c[2], c[3]);
  debug_string ("position", str, fp); 
  c = get_spot_direction ();
  snprintf (str, 255, "%g %g %g %g", c[0], c[1], c[2], c[3]);
  debug_string ("spot direction", str, fp); 
  delete str;
  debug_real ("spot exponent", (double)get_spot_exponent (), fp);
  debug_real ("spot cutoff", (double)get_spot_cutoff (), fp);
  debug_real ("constant attenuation", (double)get_constant_attenuation (), fp);
  debug_real ("linear attenuation", (double)get_linear_attenuation (), fp);
  debug_real ("quadratic attenuation", (double)get_quadratic_attenuation (), fp);

LOOKUP_DEF

SAVE_DEF 
  const GLfloat *c = get_ambient ();
  write_real ((double)c[0]);
  write_real ((double)c[1]);
  write_real ((double)c[2]);
  write_real ((double)c[3]);

  c = get_diffuse ();
  write_real ((double)c[0]);
  write_real ((double)c[1]);
  write_real ((double)c[2]);
  write_real ((double)c[3]);

  c = get_specular ();
  write_real ((double)c[0]);
  write_real ((double)c[1]);
  write_real ((double)c[2]);
  write_real ((double)c[3]);

  c = get_position ();
  write_real ((double)c[0]);
  write_real ((double)c[1]);
  write_real ((double)c[2]);
  write_real ((double)c[3]);

  c = get_spot_direction ();
  write_real ((double)c[0]);
  write_real ((double)c[1]);
  write_real ((double)c[2]);

  write_real ((double)get_spot_exponent ());
  write_real ((double)get_spot_cutoff ());

  write_real ((double)get_constant_attenuation ());
  write_real ((double)get_linear_attenuation ());
  write_real ((double)get_quadratic_attenuation ());

RESTORE_DEF
  set ((GLfloat)read_real (), (GLfloat)read_real(), (GLfloat)read_real (), 
       (GLfloat)read_real(),
       (GLfloat)read_real (), (GLfloat)read_real(), (GLfloat)read_real (), 
       (GLfloat)read_real(),
       (GLfloat)read_real (), (GLfloat)read_real(), (GLfloat)read_real (), 
       (GLfloat)read_real(),
       (GLfloat)read_real (), (GLfloat)read_real(), (GLfloat)read_real (), 
       (GLfloat)read_real(),
       (GLfloat)read_real (), (GLfloat)read_real(), (GLfloat)read_real (), 
       (GLfloat)read_real (),
       (GLfloat)read_real (), 
       (GLfloat)read_real (), 
       (GLfloat)read_real (), 
       (GLfloat)read_real ());

COPY_DEF 
  const GLfloat *a = from->get_ambient ();
  const GLfloat *d = from->get_diffuse ();
  const GLfloat *s = from->get_specular ();
  const GLfloat *p = from->get_position ();
  const GLfloat *sd = from->get_spot_direction ();
  set (a[0], a[1], a[2], a[3],
       d[0], d[1], d[2], d[3],
       s[0], s[1], s[2], s[3],
       p[0], p[1], p[2], p[3],
       sd[0], sd[1], sd[2],
       from->get_spot_cutoff (),
       from->get_constant_attenuation (),
       from->get_linear_attenuation (),
       from->get_quadratic_attenuation ());

SCAN_DEF 

FIX_POINTER_DEF 

TERMINATE_DEF 

// Now implement the members specific to ASSEMBLY, and those which 
// are not part of the macros. 

LIGHT::LIGHT (double ar, double ag, double ab, double aa,
	      double dr, double dg, double db, double da,
	      double sr, double sg, double sb, double sa,
	      double x, double y, double z, double w,
	      double sx, double sy, double sz,
	      double se,
	      double sc,
	      double ca,
	      double la,
	      double qa) :
  SiL_Light ((GLfloat)ar, (GLfloat)ag, (GLfloat)ab, (GLfloat)aa,
	     (GLfloat)dr, (GLfloat)dg, (GLfloat)db, (GLfloat)da,
	     (GLfloat)sr, (GLfloat)sg, (GLfloat)sb, (GLfloat)sa,
	     (GLfloat)x, (GLfloat)y, (GLfloat)z, (GLfloat)w,
	     (GLfloat) sx, (GLfloat) sy, (GLfloat) sz, 
	     (GLfloat) se, 
	     (GLfloat) sc, 
	     (GLfloat) ca, 
	     (GLfloat) la, 
	     (GLfloat) qa)
{
  ;
}

void LIGHT::set_light (double ar, double ag, double ab, double aa,
		       double dr, double dg, double db, double da,
		       double sr, double sg, double sb, double sa,
		       double x, double y, double z, double w,
		       double sx, double sy, double sz,
		       double se,
		       double sc,
		       double ca,
		       double la,
		       double qa)
{
  set ((GLfloat)ar, (GLfloat)ag, (GLfloat)ab, (GLfloat)aa,
       (GLfloat)dr, (GLfloat)dg, (GLfloat)db, (GLfloat)da,
       (GLfloat)sr, (GLfloat)sg, (GLfloat)sb, (GLfloat)sa,
       (GLfloat)x, (GLfloat)y, (GLfloat)z, (GLfloat)w,
       (GLfloat) sx, (GLfloat) sy, (GLfloat) sz, 
       (GLfloat) se, 
       (GLfloat) sc, 
       (GLfloat) ca, 
       (GLfloat) la, 
       (GLfloat) qa);
  return;
}

// Fix up a copy of this object for a change bulletin, after the 
// object is constructed and copied memberwise. 
void LIGHT::fixup_copy (LIGHT *rollback) const 
{ 
  PARENT()::fixup_copy (rollback);
  return;
}

void LIGHT::lose () 
{  
  ENTITY::lose (); 
  return;
}

// Final record discard. 
LIGHT::~LIGHT () 
{ 
  check_destroy (); 
} 

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
