// mklight.cc: api_make_light implementation
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: mklight.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (mklight, "$Id: mklight.cc 1 2005-08-10 23:35:07Z frank $");

#include "acis/acis.hxx"

#include <kernel/kernapi/api/check.hxx>
#include <kernel/kernapi/api/api.err>
#ifdef HAVE_ACIS_5
#include <kernel/kernutil/debug/module.hxx>
#endif // HAVE_ACIS_5
#ifdef HAVE_ACIS_6
#include <baseutil/debug/module.hxx>
#endif // HAVE_ACIS_6

#include "acis/light.hxx"
#include "acis/silapi.hxx"

#define MODULE() api 
MODULE_REF(SIL);

outcome api_make_light(double ar, double ag, double ab, double aa,
		       double dr, double dg, double db, double da,
		       double sr, double sg, double sb, double sa,
		       double x, double y, double z, double w,
		       double sx, double sy, double sz,
		       double se,
		       double sc,
		       double ca,
		       double la,
		       double qa,
		       LIGHT *&light) {
  DEBUG_LEVEL(DEBUG_CALLS)
    fprintf(debug_file_ptr, "calling api_make_light\n"); 
  API_BEGIN;
  
  light = new LIGHT (ar, ag, ab, aa,
		     dr, dg, db, da,
		     sr, sg, sb, sa,
		     x, y, z, w,
		     sx, sy, sz,
		     se, sc, ca, la, qa);
  
  result = outcome (light == NULL ? API_FAILED : 0); 

  API_END;

  DEBUG_LEVEL(DEBUG_FLOW)
    fprintf(debug_file_ptr, "leaving api_make_light: %s\n", 
	    find_err_ident(result.error_number())); 

  return result; 
} 

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
