// silapi.hxx: ACIS api extension for sil
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: silapi.hxx,v 1.15 2001/04/19 15:23:41 scmfcl Exp $

#ifndef SILAPI_HXX
#define SILAPI_HXX 1

#include "acis/decl_sil.hxx"

class LIGHT;
class ENTITY;

inline bool sil_acis_check (outcome const &result, string const &name, 
			    string const &err) {
  if (result.error_number () != 0) {
    sil_msg_error (name, err);
    sil_msg_error ("acis", find_err_mess (result.error_number ()));
    return false;
  }
  return true;
}

DECL_SIL outcome api_make_light (double ar, double ag, double ab, double aa,
				 double dr, double dg, double db, double da,
				 double sr, double sg, double sb, double sa,
				 double x, double y, double z, double w,
				 double sx, double sy, double sz,
				 double se,
				 double sc,
				 double ca,
				 double la,
				 double qa,
				 LIGHT *&light);

DECL_SIL outcome api_set_material (ENTITY *e, 
				   double ar=0.7, double ag=0.7, double ab=0.7,
				   double aa=1.0,
				   double dr=0.7, double dg=0.7, double db=0.7,
				   double da=1.0,
				   double sr=0.2, double sg=0.2, double sb=0.2,
				   double sa=0.0,
				   double er=0.0, double eg=0.0, double eb=0.0,
				   double ea=0.0,
				   double ss=1.0);

unit_vector ortho (vector v);
double volume (position a, position b, position c, position d);
void section (position p, position p1, position p2, position q, BODY *b);
BODY *convex_polygon (position p, position p1, position p2, position *pts,
		      int n);
void cut_convex (position *point, int n, BODY *B);

void create_torus (position p, unit_vector u, double a, double b, BODY **B);
void create_cylinder (position p, unit_vector u, double a, double r, BODY **B);
void create_sphere (position c, position p, BODY **B);
void create_convex (position *point, int n, BODY **B);

#endif // !SILAPI_HXX

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
