// app.cc: App class implementation.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: app.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (app, "$Id: app.cc 1 2005-08-10 23:35:07Z frank $");

#include "list.hxx"
#include "gui.hxx"
#include "symbols.hxx"
#include "interface.hxx"
#include "app.hxx"
#include "module.hxx"
#include "modules.hxx"

unsigned long App::tmp_name_counter = 0UL;
App *App::this_app = NULL;

App::App (char const *nam, const char *mods, const char *path)
{
  assert (!this_app);
  gui = NULL;
  this_app = this;
  nm = new char[strlen (nam) + 1];
  strcpy ( (char *)nm, nam);
  symbols = new Symbols ();
  interface = new PluginInterface ();
  modules = new ModuleList (path);

  // Check if modules are available
  char *mod = new char[strlen (mods) + 2];
  strcpy (mod, mods);
  mod[strlen (mods) + 1] = '\0';
  char *m = mod;
  while (*m != '\0') {
    int l = 0;
    while (m[l] != ':' && m[l] != '\0')
      ++l;
    m[l] = '\0';
    if (!modules->register_module (m)) {
      char *str = new char[strlen (m) + 25];
      strcpy (str, "module ");
      strcpy (str, m);
      strcat (str, " is not available");
      sil_msg_error ("app", str);
      delete str;
    }
    m += l + 1;
  }
  delete mod;

  gui = new GUI ();
}

App::~App ()
{
  if (gui)
    delete gui;
  gui = NULL;
  if (symbols)
    delete symbols;
  if (modules)
    delete modules;
  if (nm)
    delete nm;
  if (interface)
    delete interface;
}

void App::run ()
{
  sil_msg ("SiL (" + static_cast<string> (nm) + ") Version " +
	   static_cast<string> (VERSION) + " (" +
	   static_cast<string> (TARGET_TYPE) + "," +
	   static_cast<string> (OPENGLNAME) + 
	   ")\nCopyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>\nSee http://www.langbein.org/software/sil/ for more information.\n");
  Gtk::Main::run ();
}

/// Local Variables:
/// exe-comm: "./sil"
/// compile-command: "make"
/// End:
