// base.cxx: Base module implementation.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: base.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (base, "$Id: base.cc 1 2005-08-10 23:35:07Z frank $");

#include "base/base_module.hxx"
#include "base/base.hxx"
#include "base/basetypes.hxx"

SIL_MODULE ()
{
  return new BaseModule;
}

const char *BaseModule::nm = "base";

const char *Base::type = "Base";

const char *Light::type = "Light";

BaseModule::BaseModule ()
{
}

BaseModule::~BaseModule ()
{
}

void BaseModule::draw_object (void *object, SimpleList<void> const &hirefs)
{
  if (((Base *)object)->is ("Vectors"))
    ((Vectors *)object)->gl_render (hirefs.is_member (object));
  return;
}

void BaseModule::draw_fixed_object (void *object, 
				    SimpleList<void> const &hirefs)
{
  if (((Base *)object)->is ("Light")) {
    ((Light *)object)->gl_render ();
  }
  return;
}

void BaseModule::del_object (void *object)
{
  if (object)
    delete (Base *)object;
  return;
}

void BaseModule::load_file (FILE *fp, const char *format, bool ext,
			    bool replace)
{
  sil_msg_warn ("base", "unsupported file format");
  return;
}

void BaseModule::save_file (FILE *fp, const char *format, bool ext)
{
  sil_msg_warn ("Scaner", "unsupported file format");
  return;
}

const char *BaseModule::name () const
{
  return nm;
}

void sil_base_del (Base *b)
{
  delete b;
  return;
}

void sil_base_create_default_light ()
{    
  Light *l = new Light (0.0, 0.0, 0.0, 1.0,
			1.0, 1.0, 1.0, 1.0,
			1.0, 1.0, 1.0, 1.0,
			0.0, 0.0, 10000.0, 0.0,
			0.0, 0.0, -1.0, 0.0, 180.0,
			1.0, 0.0, 0.0);
  if (!sil_add_symbol ("DefaultLight_z", (void *)l, "base", true)) {
    // That light obviously already exists (=> DefaultLight is a reserved
    // symbol name!)
    delete l;
  }
  l = new Light (0.0, 0.0, 0.0, 1.0,
		 0.5, 0.5, 0.5, 1.0,
		 0.5, 0.5, 0.5, 1.0,
		 0.0, 10000.0, 0.0, 0.0,
		 0.0, 0.0, -1.0, 0.0, 180.0,
		 1.0, 0.0, 0.0);
  if (!sil_add_symbol ("DefaultLight_y", (void *)l, "base", true)) {
    // That light obviously already exists (=> DefaultLight is a reserved
    // symbol name!)
    delete l;
  }
  l = new Light (0.0, 0.0, 0.0, 1.0,
		 0.5, 0.5, 0.5, 1.0,
		 0.5, 0.5, 0.5, 1.0,
		 10000.0, 0.0, 0.0, 0.0,
		 0.0, 0.0, -1.0, 0.0, 180.0,
		 1.0, 0.0, 0.0);
  if (!sil_add_symbol ("DefaultLight_x", (void *)l, "base", true)) {
    // That light obviously already exists (=> DefaultLight is a reserved
    // symbol name!)
    delete l;
  }
  return;
}

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
