// geometry.cxx: Geometry classes implementation.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: geometry.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (geometry, "$Id: geometry.cc 1 2005-08-10 23:35:07Z frank $");

#include "kernel/vector.hxx"
#include "kernel/geometry.hxx"
#include "kernel/topology.hxx"
#include "kernel/analyse.hxx"

string const Geometry::tid = "Geometry";
string const Space::tid = "Space";
string const Surface::tid = "Surface";
string const Plane::tid = "Plane";
string const Sphere::tid = "Sphere";
string const Cylinder::tid = "Cylinder";
string const Cone::tid = "Cone";
string const Torus::tid = "Torus";
string const Curve::tid = "Curve";
string const Straight::tid = "Straight";
string const Circle::tid = "Circle";
string const Ellipse::tid = "Ellipse";
string const SplineCurve::tid = "SplineCurve";
string const Point::tid = "Point";

bool Ellipse::is_inside (Vector3 p, Vertex *s, Vertex *e) const
{
  // Check if a point on the ellipse is inside the arc specified by s
  // and e.
  if (s && e && s->point () && e->point ()) {
    Vector3 cp = p - *cen;
    Vector3 cs = p - *s->point () -> position ();
    Vector3 ce = p - *e->point () -> position ();
    Vector3 n1 = cs * cp;
    Vector3 n2 = cp * ce;
    return (n1 % n2 > 0.0) && (n1 % *norm > 0.0);
  }
  return true;
}

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
