// modules.hxx: Module List class.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: modules.hxx,v 1.15 2001/04/19 15:23:39 scmfcl Exp $

#ifndef MODULES_HXX
#define MODULES_HXX 1

class Module;

class ModuleList {
public:
  ModuleList (const char *path);
  ~ModuleList ();
  // Handling the modules
  bool register_module (char *name);
  Module *module (char *name) const;
  // Execute functions from all modules
  void facet_init ();
  void draw_init ();
  const char *first_format_name ();
  const char *next_format_name ();
  const char *current_format_desc ();
  void load_file (FILE *fp, const char *format, bool ext, bool replace);
  void save_file (FILE *fp, const char *format, bool ext);
  const char *first_name ();
  const char *next_name ();
  void *get_ptr (const char *module);
private:
  class FileFormat {
  protected:
    FileFormat (const char *n, const char *d, Module *m, FileFormat *ne) {
      name = n;
      desc = d;
      module = m;
      next = ne;
    }
    const char *name;
    const char *desc;
    Module *module;
    FileFormat *next;
    friend ModuleList;
  };
  class ModuleElement {
  protected:
    ModuleElement (Module *m, void *p, ModuleElement *ne) {
      module = m;
      next = ne;
      ptr = p;
    };
    Module *module;
    ModuleElement *next;
    void *ptr;
    friend ModuleList;
  };
  ModuleElement *head, *window;
  FileFormat *formats;
  FileFormat *cformat;
  char *module_path;
  Module *unkown_module;
};

#endif // !MODULES_HXX

/// Local Variables:
/// exe-comm: "./sil"
/// compile-command: "make"
/// End:
