// convert.cc: sil plugin for acis/sil conversion
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: convert.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (convert, "$Id: convert.cc 1 2005-08-10 23:35:07Z frank $");

#include "interface.hxx"

#include "kernel/kernel.hxx"
#include "kernel/topology.hxx"
#include "kernel/analyse.hxx"
#include "acis/acis.hxx"
#include "acis/silapi.hxx"

void convert_acis_symbol (const Symbol *s)
{
  sil_progress_bar (true, true);

  ENTITY *e = static_cast<ENTITY *> (sil_get_symbol_data (s));
  if (is_BODY (e)) {
    const char *n = sil_get_symbol_name (s);
    sil_progress_bar_text ("Converting " + static_cast<string const> (n));
    Solid *solid = sil_convert (static_cast<BODY *> (e), false);

    if (solid) {
      char *nn = new char[strlen (n) + 7];
      strcpy (nn, "Solid_");
      strcat (nn, n);
      while (!sil_add_symbol (nn, (void *)solid, "kernel", true))
	sil_del_symbol (nn);
      char str[512];
      snprintf (str, 511,
		"Converted %s (L: %lu, S: %lu, F: %lu, l: %lu, E: %lu, V: %lu)", 
		n,
		solid->lumps () . length (),
		solid->shells () . length (),
		solid->faces () . length (),
		solid->loops () . length (),
		solid->edges () . length (),
		solid->vertices () . length ());
      sil_msg ("convert", str);
      sil_refacet ();
      sil_updated ();
    }

  }
  
  sil_progress_bar (false);

  return;
}

void convert (SimpleList<Symbol const> *sl)
{
  Symbol const *s = sl->first ();
  while (s) {
    if (sil_symbol_is (s, "acis"))
      convert_acis_symbol (s);
    s = sl->next ();
  }
  return;
}

SIL_PLUGIN ()
{
  sil_register_tool (convert, "ACIS -> SiL", "0.1",
		     "Frank C Langbein", "1999-2000 Frank C Langbein",
		     "Convert models from acis to sil.", "acis",
		     "acis:kernel");
  sil_register_tool (convert, "SiL -> ACIS", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "Convert models from sil to acis.", "kernel",
		     "kernel:acis");
  return;
}

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
