// distort.cc: distort ACIS model
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: distort.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (distort, "$Id: distort.cc 1 2005-08-10 23:35:07Z frank $");

#include "acis/acis.hxx"

#include "list.hxx"
#include "interface.hxx"
#include "acis/light.hxx"
#include "acis/silapi.hxx"
#include "base/base.hxx"

double distort_value = 0.01;

#define PM ((rand () < (RAND_MAX >> 1))?-1.0:1.0)
#define RAND (rand () / (double) RAND_MAX)
#define RANDDIST (PM * rand () * distort_value / (double) RAND_MAX)
#define RANDVAL(a) ((a) + (a) * RANDDIST)

void move_face (FACE *f)
{
  vector *v = NULL;
  SURFACE *s = f->geometry ();
  if (is_SPHERE (s)) {
    v = new vector (RAND, RAND, RAND);
  } else if (is_CONE(s)) {
    if (((CONE*)s)->radius_ratio () == 1.0)
      v = new vector (((CONE *)s)->major_axis ());
  } else if (is_PLANE(s)) {
    v = new vector (((PLANE *) s)->normal ());
  } else if (is_TORUS(s)) {
    v = new vector (((TORUS *)s)->normal ());
  }
  if (v) {
    FACE *fs[1] = {f};
    *v *= distort_value;
    vector vv (*v);
    *v += vector (RANDDIST, RANDDIST, RANDDIST);
    vv += vector (RANDDIST, RANDDIST, RANDDIST);
    sil_acis_check (api_move_faces (1, fs, 
				    rotate_transf (RANDDIST, vv) *
				    translate_transf (*v),
				    position (-10000,-10000,-10000),
				    position (10000,10000,10000)),
		    "api_move_faces", "translate face");
  }
  return;
}

void tweak_face (FACE *f)
{
  FACE *fs[1] = {f};
  SURFACE *s[1] = {f->geometry ()};
  int reverse[1];
  reverse[0] = 0;
  if (is_SPHERE (*s)) {
    SPHERE *N = (SPHERE *)*s;
    N->set_radius (RANDVAL(N->radius ()));
    sil_acis_check (api_tweak_faces (1, fs, s, reverse,
				     position (-10000,-10000,-10000),
				     position (10000,10000,10000)),
		    "api_tweak_faces", "tweak sphere");
  } else if (is_CONE(*s)) {
    if (((CONE*)*s)->radius_ratio () == 1.0) {
      CONE *N = (CONE *)*s;
      N->set_major_axis (RANDVAL(N->major_axis ()));
      double a = asin (N->sine_angle ());
      a = RANDVAL (a);
      N->set_sine_angle (sin (a));
      N->set_cosine_angle (cos (a));
      sil_acis_check (api_tweak_faces (1, fs, s, reverse,
				       position (-10000,-10000,-10000),
				       position (10000,10000,10000)),
		      "api_tweak_faces", "tweak cone");
    }
  } else if (is_TORUS(*s)) {
    TORUS *N = (TORUS *)*s;
    N->set_major_radius (RANDVAL (N->major_radius ()));
    N->set_minor_radius (RANDVAL (N->minor_radius ()));
    sil_acis_check (api_tweak_faces (1, fs, s, reverse,
				     position (-10000,-10000,-10000),
				     position (10000,10000,10000)),
		    "api_tweak_faces", "tweak torus");
  }
  return;
}

void distort_symbol (const Symbol *s)
{
  sil_progress_bar (true, true);
  sil_progress_bar_text ("Distort");

  ENTITY *e = (ENTITY *)sil_get_symbol_data (s);
  ENTITY_LIST fcs;
  sil_acis_check (api_get_faces (e, fcs),
		  "distort", "get the faces");
  
  double prog = 0.01;
  sil_progress_bar_percentage (prog);
  double step = 0.6 / fcs.count ();
  if (sil_cancel_request ()) {
    sil_progress_bar (false);
    return;
  }
  
  API_BEGIN;
  
  bool c = true;
  fcs.init ();
  FACE *f = (FACE *)fcs.next ();
  while (f && c) {
    sil_progress_bar_percentage (prog += step);
    move_face (f);
    if (sil_cancel_request ())
      c = false;
    f = (FACE *)fcs.next ();
  }
  
  if (c) {
    
    ENTITY_LIST fcs2;
    sil_acis_check (api_get_faces (e, fcs2),
		    "distort", "get the new faces.");
    sil_progress_bar_percentage (prog += 0.01);
    double step = 0.35 / fcs2.count ();
    if (sil_cancel_request ())
      c = false;
    
    if (c) {
      
      FACE **fs= new FACE*[fcs2.count ()];
      SURFACE **ss= new SURFACE*[fcs2.count ()];
      int *reverse = new int[fcs2.count ()];
      unsigned long l = 0;
      fcs2.init ();
      f = (FACE *)fcs2.next ();
      while (f && c) {
	sil_progress_bar_percentage (prog += step);
	reverse[l] = 0;
	SURFACE *s = f->geometry ();
	fs[l] = f;
	ss[l] = s;
	if (is_SPHERE (s)) {
	  SPHERE *N = (SPHERE *)s;
	  N->set_radius (RANDVAL(N->radius ()));
	  ++l;
	} else if (is_CONE(s)) {
	  if (((CONE*)s)->radius_ratio () == 1.0) {
	    CONE *N = (CONE *)s;
	    N->set_major_axis (RANDVAL(N->major_axis ()));
	    double a = asin (N->sine_angle ());
	    a = RANDVAL (a);
	    N->set_sine_angle (sin (a));
	    N->set_cosine_angle (cos (a));
	    ++l;
	  }
	} else if (is_TORUS(s)) {
	  TORUS *N = (TORUS *)s;
	  N->set_major_radius (RANDVAL(N->major_radius ()));
	  N->set_minor_radius (RANDVAL(N->minor_radius ()));
	  ++l;
	}
	if (sil_cancel_request ())
	  c = false;
	f = (FACE *)fcs2.next ();
      }
      if (sil_cancel_request ())
	c = false;
      
      if (c) {
	sil_acis_check (api_tweak_faces (1, fs, ss, reverse,
					 position (-10000,-10000,-10000),
					 position (10000,10000,10000)),
			"api_tweak_faces", "tweak faces");
	sil_progress_bar_percentage (0.99);
      }
      
      delete fs;
      delete ss;
      delete reverse;
      
    }
    

  }

  API_END;

  sil_refacet ();
  sil_progress_bar_percentage (1.0);

  sil_progress_bar (false);

  return;
}

void distort (SimpleList<Symbol const> *sl)
{
  Symbol const *s = sl->first ();
  while (s) {
    if (strcmp (sil_get_symbol_type (s), "acis") == 0)
      distort_symbol (s);
    s = sl->next ();
  }
  return;
}

SIL_PLUGIN ()
{
  // FIXME: parameter...
  sil_register_tool (&distort, "Distort", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "Distort ACIS model.", "acis", "acis");
  return;
}

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
