// icrp.cc: icrp registration
//
// Copyright (C) 2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: icrp.cc 1 2005-08-10 23:35:07Z frank $


#include "config.h"

RCSID (icrp, "$Id: icrp.cc 1 2005-08-10 23:35:07Z frank $");

#include "interface.hxx"
#include "list.hxx"
#include "xstring.hxx"
#include "xmath.hxx"
#include "scanner/scanner.hxx"
#include "scanner/points.hxx"

double max_dist = 1.0;
double derr = 0.001;
double spring_constant = 1000.0;
double minf = 0.001;
double accel_damp = 0.25;
double time_damp = 0.001;
double dt0 = 0.001;
unsigned int preiter = 2;
unsigned int max_iter1 = 100;
unsigned int max_iter2 = 1000;
bool anim = false;

class ICRPWindow;

List<ICRPWindow> icrp_windows;

class ICRPWindow : public Gtk::Window {
public:
  ICRPWindow (SimpleList<RangePoints> *P,
	      SimpleList<string> *Pn);
  ~ICRPWindow ();
  void update ();
  void close ();
private:
  void cancel ();
  void ICRPregister ();
  SimpleList<RangePoints> *Pts;
  SimpleList<string> *PtsN;
  Gtk::VBox vbox;
  Gtk::HBox titlebox, b_max_dist, b_derr, b_spring_constant, b_minf;
  Gtk::HBox b_accel_damp, b_time_damp, b_dt0, b_preiter, b_max_iter1;
  Gtk::HBox b_max_iter2, b_anim, bbox;
  Gtk::Label titlelabel, l_max_dist, l_derr, l_spring_constant, l_minf;
  Gtk::Label l_accel_damp, l_time_damp, l_dt0, l_preiter, l_max_iter1;
  Gtk::Label l_max_iter2;
  Gtk::Adjustment a_max_dist;
  Gtk::SpinButton s_max_dist;
  Gtk::Adjustment a_derr;
  Gtk::SpinButton s_derr;
  Gtk::Adjustment a_spring_constant;
  Gtk::SpinButton s_spring_constant;
  Gtk::Adjustment a_minf;
  Gtk::SpinButton s_minf;
  Gtk::Adjustment a_accel_damp;
  Gtk::SpinButton s_accel_damp;
  Gtk::Adjustment a_time_damp;
  Gtk::SpinButton s_time_damp;
  Gtk::Adjustment a_dt0;
  Gtk::SpinButton s_dt0;
  Gtk::Adjustment a_preiter;
  Gtk::SpinButton s_preiter;
  Gtk::Adjustment a_max_iter1;
  Gtk::SpinButton s_max_iter1;
  Gtk::Adjustment a_max_iter2;
  Gtk::SpinButton s_max_iter2;
  Gtk::CheckButton cb_anim;
  Gtk::Button b_icrp, b_cancel;
};

ICRPWindow::ICRPWindow (SimpleList<RangePoints> *P,
			SimpleList<string> *Pn)
  : Gtk::Window (GTK_WINDOW_TOPLEVEL),
	 vbox (),
	 titlebox (true),
	 b_max_dist (true),
	 b_derr (true),
	 b_spring_constant (true),
	 b_minf (true),
	 b_accel_damp (true),
	 b_time_damp (true),
	 b_dt0 (true),
	 b_preiter (true),
	 b_max_iter1 (true),
	 b_max_iter2 (true),
         b_anim (true),
	 bbox (true),
	 titlelabel ("ICRP Registration"),
	 l_max_dist ("Maximal distance:"),
	 l_derr ("Delta error:"),
	 l_spring_constant ("Spring constant:"),
	 l_minf ("Minimal force:"),
	 l_accel_damp ("Acceleration damp:"),
	 l_time_damp ("Time damp:"),
	 l_dt0 ("Initial time step:"),
	 l_preiter ("Pre-registrations:"),
	 l_max_iter1 ("Max spring simmulations:"),
	 l_max_iter2 ("Max spring iterations:"),
	 a_max_dist (max_dist, 0.0, 10000.0, 0.1, 1.0, 0.0),
	 s_max_dist (a_max_dist, 0.1, 5),
	 a_derr (derr, 0.0, 10000.0, 0.001, 0.01, 0.0),
	 s_derr (a_derr, 0.1, 5),
	 a_spring_constant (spring_constant, 0.0, 10000.0, 0.1, 1.0, 0.0),
	 s_spring_constant (a_spring_constant, 0.1, 2),
	 a_minf (minf, 0.0, 10000.0, 0.001, 0.01, 0.0),
	 s_minf (a_minf, 0.1, 5),
	 a_accel_damp (accel_damp, 0.0, 10000.0, 0.1, 0.01, 0.0),
	 s_accel_damp (a_accel_damp, 0.1, 5),
	 a_time_damp (time_damp, 0.0, 10000.0, 0.001, 0.01, 0.0),
	 s_time_damp (a_time_damp, 0.1, 5),
	 a_dt0 (dt0, 0.0, 10000.0, 0.001, 0.01, 0.0),
	 s_dt0(a_dt0, 0.1, 5),
	 a_preiter (preiter, 0.0, 10000.0, 1.0, 10.0, 0.0),
	 s_preiter (a_preiter, 0.1, 0),
	 a_max_iter1 (max_iter1, 0.0, 10000.0, 1.0, 10.0, 0.0),
	 s_max_iter1 (a_max_iter1, 0.1, 0),
	 a_max_iter2 (max_iter2, 0.0, 10000.0, 100.00, 10.0, 0.0),
	 s_max_iter2 (a_max_iter2, 0.1, 0),
         cb_anim ("Animate simulation"),
	 b_icrp ("ICRP"),
	 b_cancel ("Cancel")
{
  Pts = P;
  PtsN = Pn;

  titlelabel.set_justify (GTK_JUSTIFY_CENTER);

  l_max_dist.set_justify (GTK_JUSTIFY_LEFT);
  l_derr.set_justify (GTK_JUSTIFY_LEFT);
  l_spring_constant.set_justify (GTK_JUSTIFY_LEFT);
  l_minf.set_justify (GTK_JUSTIFY_LEFT);
  l_accel_damp.set_justify (GTK_JUSTIFY_LEFT);
  l_time_damp.set_justify (GTK_JUSTIFY_LEFT);
  l_dt0.set_justify (GTK_JUSTIFY_LEFT);
  l_preiter.set_justify (GTK_JUSTIFY_LEFT);
  l_max_iter1.set_justify (GTK_JUSTIFY_LEFT);
  l_max_iter2.set_justify (GTK_JUSTIFY_LEFT);

  vbox.pack_start (titlebox, false, true, 2);
  titlebox.pack_start (titlelabel, true, true, 0);

  vbox.pack_start (b_max_dist, false, true, 2);
  b_max_dist.pack_start (l_max_dist, false, true, 0);
  b_max_dist.pack_start (s_max_dist, false, true, 0);

  vbox.pack_start (b_derr, false, true, 2);
  b_derr.pack_start (l_derr, false, true, 0);
  b_derr.pack_start (s_derr, false, true, 0);

  vbox.pack_start (b_spring_constant, false, true, 2);
  b_spring_constant.pack_start (l_spring_constant, false, true, 0);
  b_spring_constant.pack_start (s_spring_constant, false, true, 0);

  vbox.pack_start (b_minf, false, true, 2);
  b_minf.pack_start (l_minf, false, true, 0);
  b_minf.pack_start (s_minf, false, true, 0);

  vbox.pack_start (b_accel_damp, false, true, 2);
  b_accel_damp.pack_start (l_accel_damp, false, true, 0);
  b_accel_damp.pack_start (s_accel_damp, false, true, 0);

  vbox.pack_start (b_time_damp, false, true, 2);
  b_time_damp.pack_start (l_time_damp, false, true, 0);
  b_time_damp.pack_start (s_time_damp, false, true, 0);

  vbox.pack_start (b_dt0, false, true, 2);
  b_dt0.pack_start (l_dt0, false, true, 0);
  b_dt0.pack_start (s_dt0, false, true, 0);

  vbox.pack_start (b_preiter, false, true, 2);
  b_preiter.pack_start (l_preiter, false, true, 0);
  b_preiter.pack_start (s_preiter, false, true, 0);

  vbox.pack_start (b_max_iter1, false, true, 2);
  b_max_iter1.pack_start (l_max_iter1, false, true, 0);
  b_max_iter1.pack_start (s_max_iter1, false, true, 0);

  vbox.pack_start (b_max_iter2, false, true, 2);
  b_max_iter2.pack_start (l_max_iter2, false, true, 0);
  b_max_iter2.pack_start (s_max_iter2, false, true, 0);
  
  vbox.pack_start (b_anim, false, true, 2);
  b_anim.pack_start (cb_anim, false, true, 0);
  cb_anim.set_active (anim);

  vbox.pack_start (bbox, false, true, 2);
  bbox.pack_start (b_icrp, false, true, 0);
  bbox.pack_start (b_cancel, false, true, 0);

  add (vbox);

  b_icrp.clicked.connect (slot (this, &ICRPWindow::ICRPregister));
  b_cancel.clicked.connect (slot (this, &ICRPWindow::cancel));

  show_all ();

  return;
}

ICRPWindow::~ICRPWindow ()
{
  if (Pts)
    delete Pts;
  if (PtsN) {
    PtsN->purge ();
    delete PtsN;
  }
  ListNode<ICRPWindow> *n = icrp_windows.find (this);
  if (n) {
    n->unlink ();
    delete n;
  }
  return;
}

void ICRPWindow::update ()
{
  string *n = PtsN->first ();
  while (n) {
    const Symbol *s = sil_find_symbol (n->c_str ());
    if (!s) {
      ListNode<ICRPWindow> *n = icrp_windows.find (this);
      if (n) {
	n->unlink ();
	delete n;
      }
      hide_all ();
      destroy ();
    }
    n = PtsN->next ();
  }
  return;
}

void ICRPWindow::close ()
{
  ListNode<ICRPWindow> *n = icrp_windows.find (this);
  if (n) {
    n->unlink ();
    delete n;
  }
  hide_all ();
  destroy ();
  return;
}

void ICRPWindow::cancel ()
{
  close ();
  return;
}

void ICRPWindow::ICRPregister ()
{
  b_icrp.set_sensitive (false);
  b_cancel.set_sensitive (false);

  sil_progress_bar (true, true);
  sil_start_time ();

  max_dist = s_max_dist.get_value_as_float ();
  derr = s_derr.get_value_as_float ();
  spring_constant = s_spring_constant.get_value_as_float ();
  minf = s_minf.get_value_as_float ();
  accel_damp = s_accel_damp.get_value_as_float ();
  time_damp = s_time_damp.get_value_as_float ();
  dt0= s_dt0.get_value_as_float ();
  preiter = s_preiter.get_value_as_int ();
  max_iter1 = s_max_iter1.get_value_as_int ();
  max_iter2 = s_max_iter2.get_value_as_int ();
  anim = cb_anim.get_active ();

  sil_icrp (Pts, max_dist, derr, spring_constant, minf, accel_damp,
	    time_damp, dt0, preiter, max_iter1, max_iter2, anim);
  
  unsigned long sec;
  unsigned int msec;
  sil_measure_time (sec, msec);
  sil_progress_bar_assign (0.0, 1.0);
  sil_progress_bar_percentage (1.0);

  sil_msg ("ICRP", "Registered point clouds in " +
	   as_string (static_cast<unsigned long> (sec / 60)) + "m " + 
	   as_string (static_cast<unsigned long> (sec % 60)) + "." +
	   as_string (static_cast<unsigned long> (msec)) + "s");

  sil_redraw ();
  sil_updated ();
  sil_progress_bar (false);
  close ();

  return;
}

void icrp_symbol (SimpleList<Symbol const> *sl)
{
  SimpleList<RangePoints> *ps = new SimpleList<RangePoints>;
  SimpleList<string> *psn = new SimpleList<string>;
  Symbol const *s = sl->first ();
  while (s) {
    const char *t = sil_get_symbol_type (s);
    if (strcmp(t, "scanner") == 0) {
      Scanner *e = static_cast<Scanner *> (sil_get_symbol_data (s));
      if (e->is ("RangePoints")) {
	ps->add_first (static_cast<RangePoints *> (e));
	psn->add_first (new string (sil_get_symbol_name (s)));
      }
    }
    s = sl->next ();
  }

  if (ps->length () > 1) {
    ICRPWindow *fw = manage (new ICRPWindow (ps, psn));
    icrp_windows.add_first (fw);
  } else {
    delete ps;
    psn->purge ();
    delete psn;
  }
  return;
}

void update_handler ()
{
  ListNode<ICRPWindow> *l = icrp_windows.first ();
  while (l) {
    ListNode<ICRPWindow> *nn = l->next ();
    l->data () -> update ();
    l = nn;
  }
  return;
}

SIL_PLUGIN ()
{
  sil_register_tool (&icrp_symbol, "ICRP", "0.1",
		     "Frank C Langbein", "2001 Frank C Langbein",
		     "ICRP Registration for a point cloud.",
		     "scanner", "scanner");
  sil_register_update_handler (&update_handler);
  return;
}

SIL_CLOSE_PLUGIN ()
{
  sil_deregister_update_handler (&update_handler);
  ListNode<ICRPWindow> *l = icrp_windows.first ();
  while (l) {
    ListNode<ICRPWindow> *nn = l->next ();
    l->data () -> close ();
    l = nn;
  }
  return;
}


/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
