// symbols.cc: sil symbols plugin.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: symbols.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (symbols, "$Id: symbols.cc 1 2005-08-10 23:35:07Z frank $");

#include "acis/acis.hxx"

#include "list.hxx"
#include "xmath.hxx"
#include "interface.hxx"
#include "acis/silapi.hxx"
#include "material.hxx"
#include "acis/at_material.hxx"
#include "scanner/points.hxx"
#include "acis/light.hxx"

class MaterialWindow;
class TransformWindow;

List<MaterialWindow> material_windows;
List<TransformWindow> transform_windows;

class TransformWindow : public Gtk::Window {
public:
  TransformWindow (const char *name);
  ~TransformWindow ();
  void centre ();
  void ok ();
  void apply ();
  void apply_trans ();
  void close ();
  void update ();
private:
  char *symbol_name;
  Gtk::Table *table;
  Gtk::Label *l_name;
  Gtk::Label *l_ro;
  Gtk::Label *l_tr;
  Gtk::Label *l_x;
  Gtk::Label *l_y;
  Gtk::Label *l_z;
  Gtk::Label *l_p;
  Gtk::Label *l_t;
  Gtk::Label *l_s;
  Gtk::Adjustment *adj_r[3];
  Gtk::Adjustment *adj_t[3];
  Gtk::SpinButton *sb_r[3];
  Gtk::SpinButton *sb_t[3];
  Gtk::Button *b_ok;
  Gtk::Button *b_apply;
  Gtk::Button *b_apply_trans;
  Gtk::Button *b_close;
  Gtk::Button *b_centre;
};

TransformWindow::TransformWindow (const char *name) : 
  Gtk::Window (GTK_WINDOW_TOPLEVEL) 
{
  set_title ("SiL Transform Object");

  table = new Gtk::Table (6, 4, true);
  table->set_row_spacings (2);
  table->set_col_spacings (2);
  table->set_border_width (4);
  add (*table);

  symbol_name = new char[strlen(name) + 1];
  strcpy (symbol_name, name);
  l_name = new Gtk::Label (symbol_name);
  l_ro = new Gtk::Label ("Rotation");
  l_tr = new Gtk::Label ("Translation");
  l_p = new Gtk::Label ("Phi (1st z)");
  l_t = new Gtk::Label ("Theta (x)");
  l_s = new Gtk::Label ("Psi (2nd z)");
  l_x = new Gtk::Label ("x");
  l_y = new Gtk::Label ("y");
  l_z = new Gtk::Label ("z");
  table->attach (*l_name, 
		 0, 1, 0, 1,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_ro, 
		 0, 1, 1, 2,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_tr, 
		 0, 1, 3, 4,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_p, 
		 1, 2, 0, 1,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_t, 
		 2, 3, 0, 1,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_s, 
		 3, 4, 0, 1,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_x, 
		 1, 2, 2, 3,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_y, 
		 2, 3, 2, 3,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_z, 
		 3, 4, 2, 3,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);

  const Symbol *s = sil_find_symbol (symbol_name);
  double rx, ry, rz, tx, ty, tz;
  if (strcmp (sil_get_symbol_type (s), "scanner") == 0) {
    Scanner *S = (Scanner *)sil_get_symbol_data (s);
    if (S->is ("RangePoints")) {
      RangePoints *P = (RangePoints *)S;
      P->translation (tx, ty, tz);
      P->rotation (rx, ry, rz);
      rx *= 180.0 / PI;
      ry *= 180.0 / PI;
      rz *= 180.0 / PI;
    } else {
      assert (false);
    }
  } else {
    assert (false);
  }

  adj_r[0] = new Gtk::Adjustment (rx,
				  -180.0, 180.0,
				  1.0, 5.0,
				  0.0);
  sb_r[0] = new Gtk::SpinButton (*adj_r[0], 1.0, 5);
  adj_r[1] = new Gtk::Adjustment (ry,
				  -90.0, 90.0,
				  1.0, 5.0,
				  0.0);
  sb_r[1] = new Gtk::SpinButton (*adj_r[1], 1.0, 5);
  adj_r[2] = new Gtk::Adjustment (rz,
				  -180.0, 180.0,
				  1.0, 5.0,
				  0.0);
  sb_r[2] = new Gtk::SpinButton (*adj_r[2], 1.0, 5);

  adj_t[0] = new Gtk::Adjustment (tx,
				  -10000.0, 10000.0,
				  1.0, 5.0,
				  0.0);
  sb_t[0] = new Gtk::SpinButton (*adj_t[0], 1.0, 5);
  adj_t[1] = new Gtk::Adjustment (ty,
				  -10000.0, 10000.0,
				  1.0, 5.0,
				  0.0);
  sb_t[1] = new Gtk::SpinButton (*adj_t[1], 1.0, 5);
  adj_t[2] = new Gtk::Adjustment (tz,
				  -10000.0, 10000.0,
				  1.0, 5.0,
				  0.0);
  sb_t[2] = new Gtk::SpinButton (*adj_t[2], 1.0, 5);
  
  table->attach (*sb_r[0], 
		 1, 2, 1, 2,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_r[1], 
		 2, 3, 1, 2,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_r[2], 
		 3, 4, 1, 2,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  
  table->attach (*sb_t[0], 
		 1, 2, 3, 4,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_t[1], 
		 2, 3, 3, 4,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_t[2], 
		 3, 4, 3, 4,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);

  b_centre = new Gtk::Button ("Center");
  b_ok = new Gtk::Button ("OK");
  b_apply = new Gtk::Button ("Apply");
  b_apply_trans = new Gtk::Button ("Transform");
  b_close = new Gtk::Button ("Cancel");

  table->attach (*b_apply_trans, 
		 0, 2, 4, 5,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*b_centre, 
		 2, 4, 4, 5,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*b_ok, 
		 0, 1, 5, 6,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*b_apply, 
		 1, 3, 5, 6,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*b_close, 
		 3, 4, 5, 6,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);

  b_centre->clicked.connect (slot (this, &TransformWindow::centre));
  b_ok->clicked.connect (slot (this, &TransformWindow::ok));
  b_apply->clicked.connect (slot (this, &TransformWindow::apply));
  b_apply_trans->clicked.connect (slot (this, &TransformWindow::apply_trans));
  b_close->clicked.connect (slot (this, &TransformWindow::close));
  destroy.connect (slot (this, &TransformWindow::close));

  show_all ();
}

TransformWindow::~TransformWindow ()
{
  ListNode<TransformWindow> *n = transform_windows.find (this);
  n->unlink ();
  delete n;
  hide_all ();

  if (table)
    delete table;
  if (l_name)
    delete l_name;
  if (l_ro)
    delete l_ro;
  if (l_tr)
    delete l_tr;
  if (l_x)
    delete l_x;
  if (l_y)
    delete l_y;
  if (l_z)
    delete l_z;
  if (l_p)
    delete l_p;
  if (l_t)
    delete l_t;
  if (l_s)
    delete l_s;

  for (int i = 0; i < 3; ++i) {
    if (adj_r[i])
      delete adj_r[i];
    if (sb_r[i])
      delete sb_r[i];
    if (adj_t[i])
      delete adj_t[i];
    if (sb_t[i])
      delete sb_t[i];
  }
  if (b_ok)
    delete b_ok;
  if (b_apply)
    delete b_apply;
  if (b_apply_trans)
    delete b_apply_trans;
  if (b_close)
    delete b_close;
  if (symbol_name)
    delete symbol_name;
}

void TransformWindow::centre ()
{
  const Symbol *s = sil_find_symbol (symbol_name);
  if (sil_symbol_is (s, "scanner")) {
    Scanner *S = (Scanner *)sil_get_symbol_data (s);
    if (S->is ("RangePoints"))
      ((RangePoints *)S)->centre ();
  }
  sil_redraw ();
  sil_updated ();
  return;
}

void TransformWindow::ok ()
{
  apply ();
  close ();
  return;
}

void TransformWindow::apply ()
{
  const Symbol *s = sil_find_symbol (symbol_name);
  if (sil_symbol_is (s, "scanner")) {
    Scanner *S = (Scanner *)sil_get_symbol_data (s);
    if (S->is ("RangePoints")) {
      RangePoints *P = (RangePoints *)S;
      P->set_rotation (sb_r[0]->get_value_as_float () / 180.0 * PI,
		       sb_r[1]->get_value_as_float () / 180.0 * PI,
		       sb_r[2]->get_value_as_float () / 180.0 * PI);
      P->set_translation (sb_t[0]->get_value_as_float (),
			  sb_t[1]->get_value_as_float (),
			  sb_t[2]->get_value_as_float ());
    }
  }
  sil_redraw ();
  sil_updated ();
  return;
}

void TransformWindow::apply_trans ()
{
  const Symbol *s = sil_find_symbol (symbol_name);
  if (sil_symbol_is (s, "scanner")) {
    Scanner *S = (Scanner *)sil_get_symbol_data (s);
    if (S->is ("RangePoints")) {
      RangePoints *P = (RangePoints *)S;
      P->set_rotation (sb_r[0]->get_value_as_float () / 180.0 * PI,
                       sb_r[1]->get_value_as_float () / 180.0 * PI,
                       sb_r[2]->get_value_as_float () / 180.0 * PI);
      P->set_translation (sb_t[0]->get_value_as_float (),
                          sb_t[1]->get_value_as_float (),
                          sb_t[2]->get_value_as_float ());
      P->apply_transformation ();
      update ();
    }
  }
  sil_redraw ();
  sil_updated ();
  return;
}

void TransformWindow::close ()
{
  hide_all ();
  destroy ();
  return;
}

void TransformWindow::update ()
{
  const Symbol *s = sil_find_symbol (symbol_name);
  if (s) {
    if (sil_symbol_is (s, "scanner")) {
      Scanner *S = (Scanner *)sil_get_symbol_data (s);
      if (S->is ("RangePoints")) {
	double rx, ry, rz, tx, ty, tz;
        RangePoints *P = (RangePoints *)S;
	P->translation (tx, ty, tz);
	P->rotation (rx, ry, rz);
	rx *= 180.0 / PI;
	ry *= 180.0 / PI;
	rz *= 180.0 / PI;
	sb_r[0]->set_value (rx);
 	sb_r[1]->set_value (ry);
	sb_r[2]->set_value (rz);
	sb_t[0]->set_value (tx);
	sb_t[1]->set_value (ty);
	sb_t[2]->set_value (tz);
      } else {
	close ();
      }
    } else {
      close ();
    }
  } else {
    close ();
  }
  return;
}

class MaterialWindow : public Gtk::Window {
public:
  MaterialWindow (const char *name);
  ~MaterialWindow ();
  void update ();
  void ok ();
  void apply ();
  void close ();
private:
  char *symbol_name;
  Gtk::Table *table;
  Gtk::Label *l_name;
  Gtk::Label *l_r;
  Gtk::Label *l_g;
  Gtk::Label *l_b;
  Gtk::Label *l_a;
  Gtk::Label *l_colors[5];
  Gtk::Adjustment *adj_a[4];
  Gtk::Adjustment *adj_d[4];
  Gtk::Adjustment *adj_s[4];
  Gtk::Adjustment *adj_e[4];
  Gtk::Adjustment *adj_ss;
  Gtk::SpinButton *sb_a[4];
  Gtk::SpinButton *sb_d[4];
  Gtk::SpinButton *sb_s[4];
  Gtk::SpinButton *sb_e[4];
  Gtk::SpinButton *sb_ss;
  Gtk::Button *b_ok;
  Gtk::Button *b_apply;
  Gtk::Button *b_close;
};

MaterialWindow::MaterialWindow (const char *name) : 
  Gtk::Window (GTK_WINDOW_TOPLEVEL) 
{
  set_title ("SiL Edit Material");

  table = new Gtk::Table (7, 5, true);
  table->set_row_spacings (2);
  table->set_col_spacings (2);
  table->set_border_width (4);
  add (*table);

  symbol_name = new char[strlen(name) + 1];
  strcpy (symbol_name, name);
  l_name = new Gtk::Label (symbol_name);
  l_r = new Gtk::Label ("red");
  l_g = new Gtk::Label ("green");
  l_b = new Gtk::Label ("blue");
  l_a = new Gtk::Label ("alpha");
  table->attach (*l_name, 
		 0, 1, 0, 1,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_r, 
		 1, 2, 0, 1,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_g, 
		 2, 3, 0, 1,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_b, 
		 3, 4, 0, 1,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_a, 
		 4, 5, 0, 1,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);

  l_colors[0] = new Gtk::Label ("ambient");
  l_colors[1] = new Gtk::Label ("diffuse");
  l_colors[2] = new Gtk::Label ("specular");
  l_colors[3] = new Gtk::Label ("emission");
  l_colors[4] = new Gtk::Label ("shininess");

  table->attach (*l_colors[0], 
		 0, 1, 1, 2,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_colors[1], 
		 0, 1, 2, 3,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_colors[2], 
		 0, 1, 3, 4,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_colors[3], 
		 0, 1, 4, 5,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*l_colors[4], 
		 0, 1, 5, 6,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);

  const Symbol *s = sil_find_symbol (symbol_name);
  const GLfloat *ambient = NULL;
  const GLfloat *diffuse = NULL;
  const GLfloat *specular = NULL;
  const GLfloat *emission = NULL;
  const GLfloat *shininess = NULL;
#ifdef HAVE_ACIS
  if (sil_symbol_is (s, "acis")) {
    ENTITY *e = (ENTITY *)sil_get_symbol_data (s);
    ATTRIB_MATERIAL *a = (ATTRIB_MATERIAL *) 
      find_attrib (e, ATTRIB_SIL_TYPE, ATTRIB_MATERIAL_TYPE);
    if (!a) {
      api_set_material (e);
      a = (ATTRIB_MATERIAL *) 
	find_attrib (e, ATTRIB_SIL_TYPE, ATTRIB_MATERIAL_TYPE);
    }
    ambient = a->get_ambient ();
    diffuse = a->get_diffuse ();
    specular = a->get_specular ();
    emission = a->get_emission ();
    shininess = a->get_shininess ();
  } else 
#endif // HAVE_ACIS
  if (sil_symbol_is (s, "scanner")) {
    RangePoints *p = (RangePoints *)sil_get_symbol_data (s);
    Material *a = p->material ();
    if (!a) {
      a = new Material (0.7, 0.7, 0.7, 1.0,
			0.7, 0.7, 0.7, 1.0,
			0.2, 0.2, 0.2, 0.0,
			0.3, 0.3, 0.3, 0.0,
			1.0);
      p->set_material (a);
    }
    ambient = a->get_ambient ();
    diffuse = a->get_diffuse ();
    specular = a->get_specular ();
    emission = a->get_emission ();
    shininess = a->get_shininess ();
  }
  
  adj_a[0] = new Gtk::Adjustment (ambient[0],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_a[0] = new Gtk::SpinButton (*adj_a[0], 1.0, 2);
  adj_a[1] = new Gtk::Adjustment (ambient[1],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_a[1] = new Gtk::SpinButton (*adj_a[1], 1.0, 2);
  adj_a[2] = new Gtk::Adjustment (ambient[2],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_a[2] = new Gtk::SpinButton (*adj_a[2], 1.0, 2);
  adj_a[3] = new Gtk::Adjustment (ambient[3],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_a[3] = new Gtk::SpinButton (*adj_a[3], 1.0, 2);

  adj_d[0] = new Gtk::Adjustment (diffuse[0],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_d[0] = new Gtk::SpinButton (*adj_d[0], 1.0, 2);
  adj_d[1] = new Gtk::Adjustment (diffuse[1],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_d[1] = new Gtk::SpinButton (*adj_d[1], 1.0, 2);
  adj_d[2] = new Gtk::Adjustment (diffuse[2],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_d[2] = new Gtk::SpinButton (*adj_d[2], 1.0, 2);
  adj_d[3] = new Gtk::Adjustment (diffuse[3],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_d[3] = new Gtk::SpinButton (*adj_d[3], 1.0, 2);
  
  adj_s[0] = new Gtk::Adjustment (specular[0],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_s[0] = new Gtk::SpinButton (*adj_s[0], 1.0, 2);
  adj_s[1] = new Gtk::Adjustment (specular[1],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_s[1] = new Gtk::SpinButton (*adj_s[1], 1.0, 2);
  adj_s[2] = new Gtk::Adjustment (specular[2],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_s[2] = new Gtk::SpinButton (*adj_s[2], 1.0, 2);
  adj_s[3] = new Gtk::Adjustment (specular[3],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_s[3] = new Gtk::SpinButton (*adj_s[3], 1.0, 2);

  adj_e[0] = new Gtk::Adjustment (emission[0],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_e[0] = new Gtk::SpinButton (*adj_e[0], 1.0, 2);
  adj_e[1] = new Gtk::Adjustment (emission[1],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_e[1] = new Gtk::SpinButton (*adj_e[1], 1.0, 2);
  adj_e[2] = new Gtk::Adjustment (emission[2],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_e[2] = new Gtk::SpinButton (*adj_e[2], 1.0, 2);
  adj_e[3] = new Gtk::Adjustment (emission[3],
				 0.0, 1.0,
				 0.05, 0.1,
				 0.0);
  sb_e[3] = new Gtk::SpinButton (*adj_e[3], 1.0, 2);

  adj_ss = new Gtk::Adjustment (*shininess,
				  0.0, 128.0,
				  1.0, 5.0,
				  0.0);
  sb_ss = new Gtk::SpinButton (*adj_ss, 1.0, 2);

  table->attach (*sb_a[0], 
		 1, 2, 1, 2,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_a[1], 
		 2, 3, 1, 2,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_a[2], 
		 3, 4, 1, 2,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_a[3], 
		 4, 5, 1, 2,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);

  table->attach (*sb_d[0], 
		 1, 2, 2, 3,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_d[1], 
		 2, 3, 2, 3,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_d[2], 
		 3, 4, 2, 3,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_d[3], 
		 4, 5, 2, 3,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);

  table->attach (*sb_s[0], 
		 1, 2, 3, 4,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_s[1], 
		 2, 3, 3, 4,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_s[2], 
		 3, 4, 3, 4,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_s[3], 
		 4, 5, 3, 4,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);

  table->attach (*sb_e[0], 
		 1, 2, 4, 5,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_e[1], 
		 2, 3, 4, 5,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_e[2], 
		 3, 4, 4, 5,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*sb_e[3], 
		 4, 5, 4, 5,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);

  table->attach (*sb_ss, 
		 1, 2, 5, 6,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);

  b_ok = new Gtk::Button ("OK");
  b_apply = new Gtk::Button ("Apply");
  b_close = new Gtk::Button ("Cancel");

  table->attach (*b_ok, 
		 1, 2, 6, 7,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*b_apply, 
		 2, 3, 6, 7,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);
  table->attach (*b_close, 
		 3, 4, 6, 7,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		 (gint)0, (gint)0);

  b_ok->clicked.connect (slot (this, &MaterialWindow::ok));
  b_apply->clicked.connect (slot (this, &MaterialWindow::apply));
  b_close->clicked.connect (slot (this, &MaterialWindow::close));
  destroy.connect (slot (this, &MaterialWindow::close));

  l_name->show ();
  for (int i =0; i<5; ++i) {
    l_colors[i]->show ();
  }
  for (int i=0; i<4; ++i) {
    if (sb_a[i])
      sb_a[i]->show ();
  }
  for (int i=0; i<4; ++i) {
    if (sb_d[i])
      sb_d[i]->show ();
  }
  for (int i=0; i<4; ++i) {
    if (sb_s[i])
      sb_s[i]->show ();
  }
  for (int i=0; i<4; ++i) {
    if (sb_e[i])
      sb_e[i]->show();
  }

  show_all ();
}

MaterialWindow::~MaterialWindow ()
{
  ListNode<MaterialWindow> *n = material_windows.find (this);
  n->unlink ();
  delete n;
  for (int i =0; i<5; ++i) {
    l_colors[i]->hide ();
  }
  for (int i=0; i<4; ++i) {
    if (sb_a[i])
      sb_a[i]->hide ();
  }
  for (int i=0; i<4; ++i) {
    if (sb_d[i])
      sb_d[i]->hide ();
  }
  for (int i=0; i<4; ++i) {
    if (sb_s[i])
      sb_s[i]->hide ();
  }
  for (int i=0; i<4; ++i) {
    if (sb_e[i])
      sb_e[i]->hide();
  }
  if (sb_ss)
    sb_ss->hide ();
  l_name->hide ();

  if (table)
    delete table;
  if (l_name)
    delete l_name;
  for (int i =0; i<5; ++i) {
    if (l_colors[i])
      delete l_colors[i];
  }
  for (int i=0; i<4; ++i) {
    if (adj_a[i])
      delete adj_a[i];
    if (sb_a[i])
      delete sb_a[i];
  }
  for (int i=0; i<4; ++i) {
    if (adj_d[i])
      delete adj_d[i];
    if (sb_d[i])
      delete sb_d[i];
  }
  for (int i=0; i<4; ++i) {
    if (adj_s[i])
      delete adj_s[i];
    if (sb_s[i])
      delete sb_s[i];
  }
  for (int i=0; i<4; ++i) {
    if (adj_e[i])
      delete adj_e[i];
    if (sb_e[i])
      delete sb_e[i];
  }
  if (adj_ss)
    delete adj_ss;
  if (sb_ss)
    delete sb_ss;
  if (b_ok)
    delete b_ok;
  if (b_apply)
    delete b_apply;
  if (b_close)
    delete b_close;
  if (symbol_name)
    delete symbol_name;
}

void MaterialWindow::ok ()
{
  apply ();
  close ();
  return;
}

void MaterialWindow::apply ()
{
  const Symbol *s = sil_find_symbol (symbol_name);
  if (sil_symbol_is (s, "acis")) {
    ENTITY *e = (ENTITY *)sil_get_symbol_data (s);
    api_set_material (e,
		      sb_a[0]->get_value_as_float (),
		      sb_a[1]->get_value_as_float (),
		      sb_a[2]->get_value_as_float (),
		      sb_a[3]->get_value_as_float (),
		      sb_d[0]->get_value_as_float (),
		      sb_d[1]->get_value_as_float (),
		      sb_d[2]->get_value_as_float (),
		      sb_d[3]->get_value_as_float (),
		      sb_s[0]->get_value_as_float (),
		      sb_s[1]->get_value_as_float (),
		      sb_s[2]->get_value_as_float (),
		      sb_s[3]->get_value_as_float (),
		      sb_e[0]->get_value_as_float (),
		      sb_e[1]->get_value_as_float (),
		      sb_e[2]->get_value_as_float (),
		      sb_e[3]->get_value_as_float (),
		      sb_ss->get_value_as_float ());
  } else if (sil_symbol_is (s, "scanner")) {
    RangePoints *p = (RangePoints *)sil_get_symbol_data (s);
    Material *a = new Material(sb_a[0]->get_value_as_float (),
			       sb_a[1]->get_value_as_float (),
			       sb_a[2]->get_value_as_float (),
			       sb_a[3]->get_value_as_float (),
			       sb_d[0]->get_value_as_float (),
			       sb_d[1]->get_value_as_float (),
			       sb_d[2]->get_value_as_float (),
			       sb_d[3]->get_value_as_float (),
			       sb_s[0]->get_value_as_float (),
			       sb_s[1]->get_value_as_float (),
			       sb_s[2]->get_value_as_float (),
			       sb_s[3]->get_value_as_float (),
			       sb_e[0]->get_value_as_float (),
			       sb_e[1]->get_value_as_float (),
			       sb_e[2]->get_value_as_float (),
			       sb_e[3]->get_value_as_float (),
			       sb_ss->get_value_as_float ());
    p->set_material (a);
  }
  sil_redraw ();
  sil_updated ();
  return;
}

void MaterialWindow::close ()
{
  hide_all ();
  destroy ();
  return;
}

void MaterialWindow::update ()
{
  const Symbol *s = sil_find_symbol (symbol_name);
  if (s) {
    const GLfloat *ambient = NULL;
    const GLfloat *diffuse = NULL;
    const GLfloat *specular = NULL;
    const GLfloat *emission = NULL;
    const GLfloat *shininess = NULL;
#ifdef HAVE_ACIS
    if (sil_symbol_is (s, "acis")) {
      ENTITY *e = (ENTITY *)sil_get_symbol_data (s);
      ATTRIB_MATERIAL *a = (ATTRIB_MATERIAL *) 
	find_attrib (e, ATTRIB_SIL_TYPE, ATTRIB_MATERIAL_TYPE);
      if (!a) {
	api_set_material (e);
	a = (ATTRIB_MATERIAL *) 
	  find_attrib (e, ATTRIB_SIL_TYPE, ATTRIB_MATERIAL_TYPE);
      }
      ambient = a->get_ambient ();
      diffuse = a->get_diffuse ();
      specular = a->get_specular ();
      emission = a->get_emission ();
      shininess = a->get_shininess ();
    } else 
#endif HAVE_ACIS
    if (sil_symbol_is (s, "scanner")) {
      RangePoints *p = (RangePoints *)sil_get_symbol_data (s);
      Material *a = p->material ();
      if (!a) {
	a = new Material (0.7, 0.7, 0.7, 1.0,
			  0.7, 0.7, 0.7, 1.0,
			  0.2, 0.2, 0.2, 0.0,
			  0.3, 0.3, 0.3, 0.0,
			  1.0);
	p->set_material (a);
      }
      ambient = a->get_ambient ();
      diffuse = a->get_diffuse ();
      specular = a->get_specular ();
      emission = a->get_emission ();
      shininess = a->get_shininess ();
    }
    sb_a[0]->set_value (ambient[0]);
    sb_a[1]->set_value (ambient[1]);
    sb_a[2]->set_value (ambient[2]);
    sb_a[3]->set_value (ambient[3]);
    sb_d[0]->set_value (diffuse[0]);
    sb_d[1]->set_value (diffuse[1]);
    sb_d[2]->set_value (diffuse[2]);
    sb_d[3]->set_value (diffuse[3]);
    sb_s[0]->set_value (specular[0]);
    sb_s[1]->set_value (specular[1]);
    sb_s[2]->set_value (specular[2]);
    sb_s[3]->set_value (specular[3]);
    sb_e[0]->set_value (emission[0]);
    sb_e[1]->set_value (emission[1]);
    sb_e[2]->set_value (emission[2]);
    sb_e[3]->set_value (emission[3]);
    sb_ss->set_value (*shininess);
  } else {
    close ();
  }
  return;
}

void show_symbol (SimpleList<Symbol const> *sl)
{
  Symbol const *s = sl->first ();
  while (s) {
    if (sil_symbol_is_visible (s))
      sil_hide_symbol ((Symbol *)s);
    else
      sil_show_symbol ((Symbol *)s);
    s = sl->next ();
  }
  sil_redraw ();
  sil_updated ();
  return;
}

void remove_symbol (SimpleList<Symbol const> *sl)
{
  Symbol const *s = sl->first ();
  while (s){
    sil_del_symbol (sil_get_symbol_name (s));
    s = sl->next ();
  }
  sil_redraw ();
  sil_updated ();
  return;
}

void edit_symbol (SimpleList<Symbol const> *sl)
{
  // FIXME: edit light sources
  Symbol const *s = sl->first ();
  while (s) {
    if ((sil_symbol_is (s, "acis") && 
	 !is_LIGHT ((ENTITY *)sil_get_symbol_data(s))) ||
	sil_symbol_is (s, "scanner")) {
      MaterialWindow *w = manage (new 
				  MaterialWindow (sil_get_symbol_name (s)));
      material_windows.add_first (w);
    }
    s = sl->next ();
  }
  return;
}

void hilite_symbol (SimpleList<Symbol const> *sl)
{
  Symbol const *s = sl->first ();
  while (s) {
    if (sil_is_hilited (s, sil_get_symbol_data (s))) 
      sil_hilite (false, sil_get_symbol_name (s), sil_get_symbol_data (s));
    else
      sil_hilite (true, sil_get_symbol_name (s), sil_get_symbol_data (s));
    s = sl->next ();
  }
  sil_redraw ();
  return;
}

void transform_symbol (SimpleList<Symbol const> *sl)
{
  // FIXME: transform acis models, etc.
  Symbol const *s = sl->first ();
  while (s) {
    if (strcmp (sil_get_symbol_type (s), "scanner") == 0) {
      Scanner *S = (Scanner *)sil_get_symbol_data (s);
      if (S->is ("RangePoints")) {
	TransformWindow *w = manage (new TransformWindow 
				     (sil_get_symbol_name (s)));
	transform_windows.add_first (w);
      }
    }
    s  = sl->next ();
  }
  return;
}

void unite_symbol (SimpleList<Symbol const> *sl)
{
  RangePoints *rp = new RangePoints ();
  bool add = false;
  string name = "";
  Symbol const *s = sl->first ();
  while (s){
    if (strcmp (sil_get_symbol_type (s), "scanner") == 0) {
      Scanner *S = (Scanner *)sil_get_symbol_data (s);
      if (S->is ("RangePoints")) {
	if (add)
	  name += "-";
	name += static_cast<string> (sil_get_symbol_name (s));
	rp->unite ((RangePoints *)S);
	add = true;
	sil_del_symbol (sil_get_symbol_name (s));
      }
    }
    s = sl->next ();
  }

  if (add) {
    char n[name.length () + 1];
    strcpy (n, name.c_str ());
    while (!sil_add_symbol (n, (void *)rp, "scanner", true))
      sil_del_symbol (n);
  } else {
    delete rp;
  }

  sil_redraw ();
  sil_updated ();

  return;
}

void stitch_symbol (SimpleList<Symbol const> *sl)
{
  Symbol const *s = sl->first ();
  while (s) {
    if (strcmp (sil_get_symbol_type (s), "acis")) {
      if (is_BODY ((ENTITY *)sil_get_symbol_data (s))) {
	BODY *B = (BODY *)sil_get_symbol_data (s);
	sil_acis_check(api_stitch (B, B, TRUE),
		       "symbols", "api_stitch");
      }
    }
    s = sl->next ();
  }
  sil_refacet ();
  return;
}

void imprint_stitch_symbol (SimpleList<Symbol const> *sl)
{
  Symbol const *s = sl->first ();
  while (s) {
    if (strcmp (sil_get_symbol_type (s), "acis")) {
      if (is_BODY ((ENTITY *)sil_get_symbol_data (s))) {
	BODY *B = (BODY *)sil_get_symbol_data (s);
	sil_acis_check(api_imprint_stitch_complete (B, B),
		       "symbols", "api_imprint_stitch_complete");
      }
    }
    s = sl->next ();
  }
  sil_refacet ();
  return;
}

void update_handler ()
{ 
  ListNode<MaterialWindow> *n = material_windows.first ();
  while (n) {
    ListNode<MaterialWindow> *nn = n->next ();
    n->data () -> update ();
    n = nn;
  }
  ListNode<TransformWindow> *tn = transform_windows.first ();
  while (tn) {
    ListNode<TransformWindow> *tnn = tn->next ();
    tn->data () -> update ();
    tn = tnn;
  }
  return;
}

SIL_PLUGIN ()
{
  sil_register_tool (&remove_symbol, "Remove", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Remove symbol.", "acis", "acis");
  sil_register_tool (&remove_symbol, "Remove", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Remove symbol.", "scanner", "scanner");
  sil_register_tool (&remove_symbol, "Remove", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Remove symbol.", "kernel", "kernel");
  sil_register_tool (&remove_symbol, "Remove", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Remove symbol.", "base", "base");
  
  sil_register_tool (&edit_symbol, "Edit", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Edit symbol.", "acis", "acis");
  sil_register_tool (&edit_symbol, "Edit", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Edit symbol.", "scanner", "scanner");
  
  sil_register_tool (&show_symbol, "Visible", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Show symbol.", "acis", "acis");
  sil_register_tool (&show_symbol, "Visible", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Show symbol.", "scanner", "scanner");
  sil_register_tool (&show_symbol, "Visible", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Show symbol.", "kernel", "kernel");
  sil_register_tool (&show_symbol, "Visible", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Show symbol.", "base", "base");

  sil_register_tool (&hilite_symbol, "HiLite", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Hilite symbol.", "acis", "acis");
  sil_register_tool (&hilite_symbol, "HiLite", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Hilite symbol.", "scanner", "scanner");
  sil_register_tool (&hilite_symbol, "HiLite", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Hilite symbol.", "kernel", "kernel");
  sil_register_tool (&hilite_symbol, "HiLite", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Hilite symbol.", "base", "base");
  
  sil_register_tool (&transform_symbol, "Transform", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Transform symbol.", "scanner", "scanner");
  sil_register_tool (&unite_symbol, "Unite", "0.1",
		     "Frank C Langbein", "2001 Frank C Langbein",
		     "Unit Point Clouds.", "scanner", "scanner");
 
  sil_register_tool (&stitch_symbol, "Stitch", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "Stitch symbol.", "acis", "acis");

  sil_register_tool (&imprint_stitch_symbol, "Imprint Stitch", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "Imprint and stirch symbol.", "acis", "acis");
 
  sil_register_update_handler (&update_handler);

  return;
}

SIL_CLOSE_PLUGIN ()
{
  sil_deregister_update_handler (&update_handler);

  ListNode<MaterialWindow> *n = material_windows.first ();
  while (n) {
    ListNode<MaterialWindow> *nn = n->next ();
    n->data () -> close ();
    n = nn;
  }
  ListNode<TransformWindow> *tn = transform_windows.first ();
  while (tn) {
    ListNode<TransformWindow> *tnn = tn->next ();
    tn->data () -> close ();
    tn = tnn;
  }

  return;
}

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
