// symbols.hxx: Symbols class.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: symbols.hxx,v 1.17 2001/04/19 15:23:40 scmfcl Exp $

#ifndef SYMBOLS_HXX
#define SYMBOLS_HXX 1

class Symbol;
class Symbols;
class Module;

class Symbol
{
public:
  Symbol (char *name, void *data, char *type, bool visible);
  ~Symbol ();
  bool is (const char *t) const;
  const char *type () const;
  inline char *name () const {
    return nm;
  }
  inline void *data () const {
    return dt;
  }
  inline bool is_visible () const {
    return visible;
  }
  void show ();
  void hide ();
  void facet () const;
  void draw () const;
  void draw_fixed () const;
  void hilite (bool hi, void *ref) {
    if (hi)
      hilite_refs->add_first (ref);
    else
      hilite_refs->del (ref);
    return;
  }
  bool is_hilited (const void *s) const {
    return hilite_refs->is_member (s);
  }
protected:
  Symbol *next;
  void *dt;
  Module *module;
  SimpleList<void> *hilite_refs;
  friend Symbols;
private:
  char *nm;
  bool visible;
};

class SymbolsUpdateHandler
{
protected:
  void (*handler) (void);
  SymbolsUpdateHandler *next;
  int use_counter;
  friend Symbols;
};

class Symbols
{
public:
  Symbols ();
  ~Symbols ();
  bool add (char *name, void *data, char *type, bool visible);
  const Symbol *find (const char *name);
  void del (const char *name);
  void clear ();
  Symbol *first ();
  Symbol *next ();
  void updated ();
  void add_update_handler (void (*f)(void));
  void del_update_handler (void (*f)(void));
  void hilite (bool hi, char const *name, void *ref);
private:
  Symbol *head;
  Symbol *window;
  SymbolsUpdateHandler *handler_head;
};

#endif // !SYMBOLS_HXX

/// Local Variables:
/// exe-comm: "./sil"
/// compile-command: "make"
/// End:
