// xmath.cc: math extensions.
// 
// Copyright (C) 1999-2001 Frank Langbein <frank@langbein.org>
// All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
// 
// $Id: xmath.cc 1 2005-08-10 23:35:07Z frank $
//

#include "config.h"

RCSID (xmath, "$Id: xmath.cc 1 2005-08-10 23:35:07Z frank $");

#include "interface.hxx"
#include "xmath.hxx"

// Normalize a 3D GLfloat vector in place.
void normalize (GLfloat *v)
{
  assert (v);
  GLfloat d = sqrt (v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
  if (fabs (d) <= EPSILON) {
    sil_msg_error ("xmath", "tried to normalize a zero length vector");
  } else {
    v[0] /= d;
    v[1] /= d;
    v[2] /= d;
  }
  return;
}

// Normalized cross product of two 3D GLfloat vectors.
void ncrossprod (const GLfloat * v1, const GLfloat * v2, GLfloat * cp)
{
  assert (v1 && v2 && cp);
  cp[0] = v1[1] * v2[2] - v1[2] * v2[1];
  cp[1] = v1[2] * v2[0] - v1[0] * v2[2];
  cp[2] = v1[0] * v2[1] - v1[1] * v2[0];
  normalize (cp);
  return;
}

/// Local Variables:
/// exe-comm: "./sil"
/// compile-command: "make"
/// End:
